/* Copyright (C) 1999 PLUS Group - University of Maryland
 *	
 * This software is provided under the terms of the GNU General Public
 * License Version 2, June 1991. See the header in KA_App.java
 * and the file License.html for details.
 */

/* Change history:
   09/25/01, JDH: Commented out do-nothing code in action() method
   10/26/00, JDH: Used accessor methods for Category and Relation fields
   */

package Annotator;

import java.awt.*;
import java.util.*;
import UserInt.*;
import SHOE.*;
import Tree.*;
import java.lang.System;

public class ClaimDlg extends WorkaroundDialog {
  // winMode constants
  public static final int ADD=1;
  public static final int EDIT=2;
  // topCard constants
  public static final int CATEGORY = 1;
  public static final int RELATION = 2;

  int topCard = CATEGORY;

  Frame parent;

  Button okButton;
  Button newButton;
  Button cancelButton;

  Choice filterChoice;
  TreeWidget useOntList;
  TreeWidget elemTree;
  
  ObjectList  claimList;
  OntManager  ontman;
  int         winMode;
  int         edIndex;
  Instance    edInst;
  Declaration edClaim;

  Panel relPanel; 
  Panel cardPanel;
  Choice typeChoice;
  TextField preText;
  TextField relNameText, argText[];
  TextField catNameText, forText;
  Label l_type, l_pre;
  Label l_relName, l_arg[];                          // relation labels
  Label l_catName, l_for;                         // category labels
  Button b_BrowseOnt, b_BrowseRel, b_BrowseCat;
  
  public ClaimDlg(Frame dw, int mode, ObjectList cList,
		  Instance inst, OntManager om){
    super(dw,"Claim",false);
    parent =dw;
    winMode=mode;
    claimList=cList;
    edInst=inst;
    ontman=om;

    // init the interface
    setup();

    // init data
    initdata();

    //Initialize this dialog to its prefered size.
    pack();
  }

  /** Creates and lays out the fields */
  void setup(){
    /* The panel containment hierarchy is:
       [this]
         topPanel
           ontPanel
	   elemPanel
	   filterPanel
	 bottomPanel
	   claimPanel
	     choicePanel
	     cardPanel
	       relPanel, catPanel
	   bnPanel
   */
    int i;

    // Build sub-panels of the top panel
	
    // Ontology Panel
    Panel ontPanel = new Panel();
    ontPanel.setLayout(new BorderLayout());
    useOntList = new TreeWidget(10, false);
    ontPanel.add("North", new Label("Use Ontologies:"));
    ontPanel.add("Center", useOntList);

    // Element Panel
    Panel elemPanel = new Panel();
    elemPanel.setLayout(new BorderLayout());
    elemTree = new TreeWidget(10, false);
    elemPanel.add("North", new Label("Elements:"));
    elemPanel.add("Center", elemTree);

    // Filter Panel    
    Panel filterPanel = new Panel();
    filterPanel.setLayout(new BorderLayout());
    filterChoice = new Choice();
    filterChoice.addItem("All");
    filterChoice.addItem("Category");
    filterChoice.addItem("Relation");
    filterPanel.add("North", new Label("Filter:"));	
    filterPanel.add("Center",filterChoice);

    // create top panel and add the Ontology, Element, and Filter panels
    Panel topPanel = new Panel();
    topPanel.setLayout(new GridLayout(1, 3));
    topPanel.add(ontPanel);
    topPanel.add(elemPanel);
    topPanel.add(filterPanel);

    /* Bottom panel consists of Claims and Buttons */

    // Choice panel includes type of claims and ontology prefix
    Panel choicePanel = new Panel();
    choicePanel.setLayout(new GridBagLayout());
    l_type = new Label("Type:");
    typeChoice = new Choice();
    typeChoice.addItem("Category");
    typeChoice.addItem("Relation");
    l_pre = new Label("Prefix:");
    preText = new TextField(40);
    preText.setBackground(Color.white);
    //    b_BrowseOnt = new Button("Set");
    Utilities.placeComponent(choicePanel, l_type, 0, 0, 1, 1);
    Utilities.placeComponent(choicePanel, typeChoice, 1, 0, 1, 1);
    Utilities.placeComponent(choicePanel, l_pre, 0, 1, 1, 1);
    Utilities.placeComponent(choicePanel, preText, 1, 1, 2, 1);
    //    Utilities.placeComponent(choicePanel, b_BrowseOnt, 3, 1, 1, 1);

    // create relation "card"
    relPanel = new Panel();    
    relPanel.setLayout(new GridBagLayout());
    l_relName = new Label("Name:");
    relNameText = new TextField(40);
    relNameText.setBackground(Color.white);
    //    b_BrowseRel = new Button("Browse");
    Utilities.placeComponent(relPanel, l_relName, 0, 0, 1, 1,
			     GridBagConstraints.EAST);
    Utilities.placeComponent(relPanel, relNameText, 1, 0, 3, 1);
    //    Utilities.placeComponent(relPanel, b_BrowseRel, 4, 0, 1, 1);
    l_arg = new Label[2];
    argText = new TextField[2];
    for (i=0; i < 2; i++) {
      l_arg[i] = new Label("Argument " + (i + 1) + ":");
      Utilities.placeComponent(relPanel, l_arg[i], 0, i+1, 1, 1,
			       GridBagConstraints.EAST);
      argText[i] = new TextField(40);
      argText[i].setBackground(Color.white);
      Utilities.placeComponent(relPanel, argText[i], 1, i+1, 3, 1);
    }

    // create category "card"
    Panel catPanel = new Panel();
    catPanel.setLayout(new GridBagLayout());
    l_catName = new Label("Name:");
    catNameText = new TextField(40);
    catNameText.setBackground(Color.white);
    l_for = new Label("For:");
    forText = new TextField(40);
    forText.setBackground(Color.white);
    //    b_BrowseCat = new Button("Browse");
    Utilities.placeComponent(catPanel, l_catName, 0, 0, 1, 1,
			     GridBagConstraints.EAST);
    Utilities.placeComponent(catPanel, catNameText, 1, 0, 3, 1);
    //    Utilities.placeComponent(catPanel, b_BrowseCat, 4, 0, 1, 1);
    Utilities.placeComponent(catPanel, l_for, 0, 1, 1, 1,
			     GridBagConstraints.EAST);
    Utilities.placeComponent(catPanel, forText, 1, 1, 3, 1);

    // create Card Panel and add category and relation "cards"
    cardPanel = new Panel();
    cardPanel.setLayout(new CardLayout());
    cardPanel.add("Category", catPanel);
    cardPanel.add("Relation", relPanel);

    // Create claim panel and add Choice and Card panels to it
    Panel claimPanel = new Panel();
    claimPanel.setLayout(new BorderLayout());
    claimPanel.add("North", choicePanel);
    claimPanel.add("Center", cardPanel);

    // Buttons Panel
    Panel bnPanel=new Panel();
    bnPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
    okButton=new Button("Ok");
    cancelButton=new Button("Cancel");
    newButton=new Button("Next");
    bnPanel.add(okButton);
    bnPanel.add(cancelButton);
    bnPanel.add(newButton);

    // create Bottom panel and add Claim and Button panels to it
    Panel bottomPanel = new Panel();
    bottomPanel.setLayout(new FlowLayout());//GridLayout(1, 2));
    bottomPanel.add(claimPanel);
    bottomPanel.add(bnPanel);

    // add the Top and Bottom panels to the dialog
    add("North",topPanel);	   
    add("South",bottomPanel);
  }


  /* Set default values for fields */
  void initdata(){

    int i;    
    RelArg argList[];
    Vector uontTreeList;

    // populate the "Use Ontology" pick list
    uontTreeList = edInst.getUseOntTreeList(ontman);
    useOntList.initTree(uontTreeList);

    if (winMode == ADD){
      setTitle("Add Claim");
      edIndex = claimList.getSelectedIndex();
      edClaim = (Declaration)claimList.getSelectedObject();

      if(edIndex==-1) {                     // no claim is selected
	filterChoice.select("Category");

	/* uncomment below for default values when no claims have been added */
	// forText.setText(edInst.getKey());
	// argText[0].setText(edInst.getKey());
      }

      /* user has selected an unnamed subject in the Claim List. We must
	 look at the next claim to determine the subject's key */
      else if(edClaim==null) {
	filterChoice.select("Category");	
      	Declaration dec = (Declaration)claimList.getObject(edIndex+1);
	if (dec instanceof Relation) {
	  Relation rel = (Relation)dec;
	  argText[0].setText(rel.getArgs()[0].getValue());
	  forText.setText(rel.getArgs()[0].getValue());
	}
	else if (dec instanceof  Category) {
	  Category cat = (Category)dec;
	  argText[0].setText(cat.getForText());
	  forText.setText(cat.getForText());
	}
	
	//argText[0].setText(claimList.getSelectedItem());
	//forText.setText(claimList.getSelectedItem());
      }
      else if (edClaim instanceof Relation) {
	Relation rel = (Relation)edClaim;
	
	/* set up according to relation definition */
	Def_Relation drel = (Def_Relation)ontman.getElement(rel.getElmtId());
	if (drel == null) {
	  /* Relation may have an error, but should allow editing */
	  resetArgLabels(rel.getArgCount());
	}
	else
	  updateArgLabels(drel);
	
	/* fill in text field values */
	preText.setText(getFullPrefix(rel.getName()));
	relNameText.setText(removeFullPrefix(rel.getName()));
	
	argList = rel.getArgs();
	int argcnt = argList.length;
	
	topCard = RELATION;
	((CardLayout)cardPanel.getLayout()).show(cardPanel, "Relation");
	typeChoice.select("Relation");
	filterChoice.select("Relation");
	
	argText[0].setText(argList[0].getValue());
	forText.setText(argList[0].getValue());
      }
      else if (edClaim instanceof  Category) {
	Category cat = (Category)edClaim;
	
	preText.setText(getFullPrefix(cat.getName()));
      	topCard = CATEGORY;
	((CardLayout)cardPanel.getLayout()).show(cardPanel, "Category");
	typeChoice.select("Category");
	filterChoice.select("Category");
	
	argText[0].setText(cat.getForText());
	forText.setText(cat.getForText());
      }	

      /* if no claim was selected, set first ontology as the default */
      if (edClaim == null) {
	if (useOntList.countItems() > 0) {       // getItemCount() in 1.1
	  useOntList.select(0);
	  /* since handle event will not fire yet, set the prefix */
	  updatePrefixField();
	}
      }
    }

    else {                                 // winMode==EDIT
      setTitle("Edit Claim");
      edIndex = claimList.getSelectedIndex();
      edClaim = (Declaration)claimList.getSelectedObject();
      
      /* it is possible for Edit to be pressed when no claim is selected */
      if(edIndex==-1) {
	setTitle("Add Claim");
	filterChoice.select("Category");
	winMode=ADD;
      }

      /* user has selected an unnamed subject in the Claim List. */
      else if(edClaim==null) {
	Unique_Elt nameId = null;
	Def_Relation nameRel;

	/* NOTE: the next two sections of code should be replaced with
	   a simple retrieval of the "name" relation from the base
	   ontology. Use the "." prefix to refer to it. */

	try {
	  nameId = ontman.resolveName(".name", edInst);
	} catch (SHOEException ex) {
	  System.out.println("System Error: Cannot locate Def-Relation 'name' in base ontology");
	}
	nameRel = (Def_Relation)ontman.getElement(nameId);
	updateArgLabels(nameRel);
	preText.setText("");
	relNameText.setText("name");
	      
	/* look at next claim to determine the default value for the
	   first argument of the new "name" relation */
      	Declaration dec = (Declaration)claimList.getObject(edIndex+1);
	if (dec instanceof Relation) {
	  Relation rel = (Relation)dec;
	  argText[0].setText(rel.getArgs()[0].getValue());
	}
	else if (dec instanceof  Category) {
	  Category cat = (Category)dec;
	  argText[0].setText(cat.getForText());
	}
	
	topCard = RELATION;
	((CardLayout)cardPanel.getLayout()).show(cardPanel, "Relation");
	typeChoice.select("Relation");
	typeChoice.disable();
	filterChoice.select("Relation");
	//showElements();
	
	winMode = ADD;
      }

      else
	fillClaimDefaults(edClaim);

    }
    showElements();                    // fill in the elements list
  }

  /** Given a claim, updates the appropriate card with its values and
     displays that card. */
  void fillClaimDefaults(Declaration edClaim) {

    int i;    
    RelArg argList[];

    if (edClaim instanceof Relation) {
      Relation rel = (Relation)edClaim;

      /* set up according to relation definition */
      Def_Relation drel = (Def_Relation)ontman.getElement(rel.getElmtId());
      if (drel == null) {
	System.out.println("Couldn't find corresponding DEF-RELATION");
	resetArgLabels(rel.getArgCount());
      }
      else
	updateArgLabels(drel);

      /* fill in text field values */
      preText.setText(getFullPrefix(rel.getName()));
      relNameText.setText(removeFullPrefix(rel.getName()));

      argList = rel.getArgs();
      int argcnt = argList.length;
      for (i=0; i < argcnt; i++)
	argText[i].setText(argList[i].getValue());

      topCard = RELATION;
      ((CardLayout)cardPanel.getLayout()).show(cardPanel, "Relation");
      typeChoice.select("Relation");
      typeChoice.disable();
      filterChoice.select("Relation");
    }
    else if (edClaim instanceof  Category) {
      Category cat = (Category)edClaim;

      preText.setText(getFullPrefix(cat.getName()));
      catNameText.setText(removeFullPrefix(cat.getName()));
      forText.setText(cat.getForText());
      
      topCard = CATEGORY;
      ((CardLayout)cardPanel.getLayout()).show(cardPanel, "Category");
      typeChoice.select("Category");
      typeChoice.disable();
      filterChoice.select("Category");
    }
  }


  /** Fills the Elements list with the categories and/or relations that
    may involve the instance that has been identified in the For or
    first argument field. */
  public void showElements(){

    String type;    
    SHOE_Obj obj;
    SHOE_Obj baseObj;        // base object, same as obj except for renames
    Enumeration eltEnum;
    Unique_Elt elmt_id;
    Ontology ont;
    Use_Ontology uont;
    int i;
    Hashtable ancestorIds = null;    /* has ancestors of the subject */
    /* An enumeration of the hashtable elements can then be used to determine
       what the allowed types are for a relation's subject when filtering */

    Vector vElem=new Vector();
    type=filterChoice.getSelectedItem();

    /* Find the ontology that has been selected */
    uont=(Use_Ontology)useOntList.getSelectedObject();
    if (uont == null)
      return;
    ont = ontman.getOntology(uont);
    if (ont == null)
      return;

    /* add category hierarchies to the selection list */
    Enumeration      catTrees;
    DefaultTreeModel nextTree;

    if(type.equals("Category") || type.equals("All")){	    
      catTrees = ont.getLabeledCategoryTreeLists(ontman);
      while (catTrees.hasMoreElements()) {
	nextTree = (DefaultTreeModel)catTrees.nextElement();
	vElem.addElement(nextTree);    
      }
    }

    /* add relation to the selection list */
    DefaultTreeModel tree;
    LabeledTreeNode  node;

    if(type.equals("Relation") || type.equals("All")){

      /* The following code is used to determine what categories the
	 instance specified as the subject belongs too. This list is used
	 to display only relations in which it can legally be the subject. */
      String strSubject=argText[0].getText();
      if(strSubject!=null && !strSubject.equals("")) {

	// put instance key in place of "ME"
	strSubject = replaceME(strSubject); 
	
	/* subject refers to a constant or category */
	if (strSubject.startsWith("!")) {
	  try {
	    elmt_id = ontman.resolveName(strSubject.substring(1), edInst);
	    if (elmt_id != null)
	      ancestorIds = ontman.getAncestorHashtable(elmt_id);
	  } catch (SHOEException ex) {
	  }
	}
	/* subject refers to an instance */
	else {
	  Enumeration enumDec = edInst.getDeclarations();
	  ancestorIds = new Hashtable();
	  /* Since we want to have the ancestors for all categories of the
	     instance in a single hashtable, we construct it manually. */
	  while (enumDec.hasMoreElements()) {
	    obj = (SHOE_Obj)enumDec.nextElement();
	    if (obj instanceof Category) {
	      Category cat=(Category)obj;
	      if(cat.getForText().equals(strSubject)) {
		Unique_Elt catId = cat.getElmtId();
		/* if the category is in error, then there may not be an id */
		if (catId != null) {
		  ancestorIds.put(catId, "");
		  ontman.putParentsInHashTable(ancestorIds, catId);
		  
		}
	      }  
	    }
	  }
	}

      /* rather than getting the entire set of declarations and searching
	 for categories that involve the subject (as above), we might
	 consider having the instance keep a hashtable of of declarations
	 that hashes on the subject and stores a vector of declarations
	 for the subject */

      }
      
      /*
	if (ancestorIds != null) {
	System.out.println("Size of table is " + ancestorIds.size());
	System.out.println("Table is:");
	System.out.println(ancestorIds);
	}
	*/


      /* cycle through the selected ontology's elements looking for
	 relations and renames in which the specified subject instance
	 is of the correct type to be used with. */
      eltEnum = ont.getSortedElements();                // for relation only
      while (eltEnum.hasMoreElements()) {
	obj = (SHOE_Obj)eltEnum.nextElement();
	baseObj = obj;
	/* in the case of renames, find the original object */
	if (obj instanceof Def_Rename) {
	  elmt_id = ((Def_Rename)obj).getFromId();
	  baseObj = ontman.getElement(elmt_id);
	}
      
	/* below we check the type of the base object to determine if
	   something is a relation or category. This way renamed relations 
	   and categories will appear in the list. When added to the list,
	   the list text comes from the ontology object so that the new
	   name of renamed objects will appear. However, the corresponding
	   object stored in the list is the base object so that the actual
	   Def-Relation can be used to find information on the arguments.
	   */
	
	if (baseObj instanceof Def_Relation) {

	  /* if subject has no parents, don't filter the relations */
	  if (ancestorIds == null) {
	    node = new LabeledTreeNode(baseObj, obj.getKey());
	    tree = new DefaultTreeModel(node);
	    vElem.addElement(tree);    
	  }
	  /* otherwise, show only the relations it matches types with */
	  else {
	    Def_Relation drel = (Def_Relation)baseObj;
	    Def_Arg subjectArg = drel.getDefArgAt(0);
	    Unique_Elt typeId = subjectArg.getTypeId();
	    // System.out.println("typeId=" + typeId);
	    if (ancestorIds.containsKey(typeId)) {
	      node = new LabeledTreeNode(baseObj, obj.getKey());
	      tree = new DefaultTreeModel(node);
	      vElem.addElement(tree);    
	      // System.out.println("added " + obj);
	    }
	  }
	}
      }
    }

    elemTree.initTree(vElem);
  }


  public String replaceME(String str) {
    if(str.equalsIgnoreCase("ME"))
      return edInst.getKey();
    else return str;
  }


  public boolean saveChanges() {

    Relation rel;
    Category cat;
    int i;
    String prefixedName;
    String argStr = null;

    switch (topCard) {
    case CATEGORY:
      if(preText.getText().equals("") || catNameText.getText().equals("")){
	notifyUser("You must fill the Prefix and Name fields!"); 
	return false;
      }

      prefixedName = preText.getText() + "." + catNameText.getText();
      if (winMode == ADD) {
	cat = new Category(prefixedName, replaceME(forText.getText()));
	/* the instance will validate the category before adding it */
	edInst.addDeclaration(cat, ontman);
	//claimList.addItem(cat.getPredicateText(), cat);
      }
      else {
	cat = (Category)edClaim;
	cat.setName(prefixedName);
	cat.setForText(replaceME(forText.getText()));

	cat.validate(edInst,ontman);

	//claimList.replaceItem(cat.getPredicateText(), cat, edIndex);
      }
      break;
    case RELATION:
      if(preText.getText().equals("") &&
	 !relNameText.getText().equals("name")) {
	notifyUser("You must fill the Prefix field!"); 
	return false;
      }
      if(relNameText.getText().equals("")){
	notifyUser("You must fill the Name field!"); 
	return false;
      }
      for(i=0; i < argText.length; i++) 
	if(argText[i].getText().equals("")) {
	  notifyUser("You must fill all the argument fields!"); 
	  return false;
	}

      prefixedName = preText.getText() + "." + relNameText.getText();
      if (winMode == ADD) {
	rel = new Relation(prefixedName);
	for (i=0; i < argText.length; i++) {
	  argStr = replaceME(argText[i].getText());
	  /* only add arguments with not-null values, defaults will be
	     supplied by the validation procedure */
	  /* note, there shouldn't be any null arguments */
	  if (argStr != null && argStr.length() > 0)
	    rel.addArg(new RelArg(i+1, argStr));
	}
	//	System.out.println("Rel is: " + rel);
	edInst.addDeclaration(rel, ontman);
	//claimList.addItem(rel.getPredicateText(), rel);
      }
      else {
	rel = (Relation)edClaim;
	rel.setName(prefixedName);
	for (i=0; i < argText.length; i++)
	  rel.setArgValue(i+1, replaceME(argText[i].getText()));

	rel.validate(edInst,ontman);
	//claimList.replaceItem(rel.getPredicateText(), rel, edIndex);
      }
      break;
    }


    Enumeration enum = edInst.getDeclarations();

    /* We replace the entire claim list because a new claim could be 
       inserted anywhere within it. The new claim should appear under
       the heading of the subject that is its first argument */
    ((KA_App)parent).replaceClaimList(enum);

    return true;
  }



  /** Responds to events that destroy the window, select the ontology
    list, select the element list, or signal that loss of focus from
    the first argument field. */
  public boolean handleEvent(Event event) {

    LabeledTreeNode node;
    SHOE_Obj obj; 

    if (event.id == Event.WINDOW_DESTROY) {
      dispose();
    }
    else if(event.id == Event.LIST_SELECT){
      if(event.target == useOntList){
	updatePrefixField();
	showElements();
	}
      else if(event.target == elemTree){
	node = elemTree.getSelectedNode();
	obj = (SHOE_Obj)node.getUserObject();
	if(obj instanceof Def_Relation){
	  relNameText.setText(node.getLabel());
	  updateArgLabels((Def_Relation)obj);
	}	  
	else if(obj instanceof Def_Category)
	  catNameText.setText(node.getLabel());
      }	
    }
    else if (event.id == Event.LOST_FOCUS) {
    /* if the user changes the first argument, recompute the relation in
       which in can participate */
      if (event.target == argText[0]) {
	// System.out.println("Arg 1 lost focus");
	showElements();
      }
    }
    return super.handleEvent(event);
  }
  


  public boolean action(Event event, Object arg){
    /* The following code does nothing and doesn't compile in Java 2
       because List is ambiguous */
    /*
    if(event.target instanceof List){
      //	if(event.target == elemTree)		showClaim();
    }
    else if (event.target instanceof TextField) {
      //	UpdateClaim(event.target);
    }
    else */

    if (event.target instanceof Choice) {
      String winName=(String)arg;

      if(event.target == typeChoice){
	if (winName.equals("Category"))
	  topCard = CATEGORY;
	else if (winName.equals("Relation"))
	  topCard = RELATION;	      
	((CardLayout)cardPanel.getLayout()).show(cardPanel,winName);
	
	String str=filterChoice.getSelectedItem();
	if(!str.equals(winName)) {
	  filterChoice.select(winName);
	  showElements();
	}
	
	return true;
      }
      else if(event.target == filterChoice){
	showElements();
	if(winMode==ADD){
	  if (winName.equals("Category"))
	    topCard = CATEGORY;
	  else if (winName.equals("Relation"))
	    topCard = RELATION;	      
	  ((CardLayout)cardPanel.getLayout()).show(cardPanel,winName);
	  typeChoice.select(winName);
	}
	return true;
      }
    }
    else if (event.target instanceof Button) {
      if(event.target == cancelButton){
	dispose();
	return true;
      }
      else if(event.target == okButton){
	Onok();		
	return true;
      }
      else if(event.target == newButton){
	Onnew();
	return true;
      }
    }
    return false;
  }

  void Onok(){
    if(saveChanges()==true)   
      dispose();
  }

  void Onnew(){
    int i;
    Category cat;
    Relation rel;

    if(saveChanges()==false)
      return;

    if (winMode == ADD) {               // add a new claim
      /* set default for card that isn't showing */
      if(topCard == RELATION)
	forText.setText(argText[0].getText());
      else
	argText[0].setText(forText.getText());
	
      /* clear out arguments and category names */
      for(i=1;i<argText.length;i++)
	argText[i].setText("");
      catNameText.setText("");
    }
    else {                          // edit next claim
      if (edIndex < claimList.countItems() - 1) {
	claimList.select(edIndex + 1);
	edIndex = claimList.getSelectedIndex();
	edClaim = (Declaration)claimList.getSelectedObject();
	fillClaimDefaults(edClaim);
	showElements();
      }
    }

    //preText.setText(useOntList.getPrefix());

    //SHOE_Obj obj;

    //obj = (SHOE_Obj)elemTree.getSelectedObject();
    //if(obj instanceof Def_Relation){
    //  relNameText.setText(obj.getKey());
    //  updateArgLabels((Def_Relation)obj);
    //}	  
   // else if(obj instanceof Def_Category)
   //   catNameText.setText(obj.getKey());	
  }
    
  /** Sets the number of argument labels and fields to a desired value.
     Labels default to "Argument num:" style */
  public void resetArgLabels(int argcnt) {

    int i;

    /* erase arguments from previous relation, if not the same number */
    if (argcnt != l_arg.length) {
      String arg1=argText[0].getText();     // remember so it can be set again
      for (i=0; i <l_arg.length; i++) {
	relPanel.remove(l_arg[i]);
	relPanel.remove(argText[i]);
      }

      l_arg = new Label[argcnt];
      argText = new TextField[argcnt];
      for (i=0; i < argcnt; i++) {
	l_arg[i] = new Label("Argument " + (i + 1) + ":");
	Utilities.placeComponent(relPanel, l_arg[i], 0, i+1, 1, 1,
				 GridBagConstraints.EAST);
	argText[i] = new TextField(40);
	argText[i].setBackground(Color.white);
	Utilities.placeComponent(relPanel, argText[i], 1, i+1, 3, 1);
      }
      argText[0].setText(arg1);     // keep value from first field
    }
    pack();
  }

  /** Updates the labels and argument text fields on the relation panel
    to correspond to the definition of the argument. */
  public void updateArgLabels(Def_Relation drel) {

    Def_Arg arglist[];
    String argLabel;
    int argcnt;
    int i;

    arglist = drel.getDefArgs();
    argcnt = arglist.length;

    resetArgLabels(argcnt);

    /* update labels with short description */
    for (i=0; i < argcnt; i++) {
      argLabel = arglist[i].getPhrase();
      if (argLabel == null)
	argLabel = "(" + arglist[i].getType() + ")";
      l_arg[i].setText(argLabel);
    }
    pack();         // this probably resizes the window if necessary
  }

  /** Sets the prefix field based upon the use ontology selected in the
    "Use Ontology" list. */
  void updatePrefixField() {

    LabeledTreeNode   selNode;
    Use_Ontology      uont;
    String            fullPrefix;
    LabeledTreeNode   parent;
    String            prefix;

    selNode = useOntList.getSelectedNode();
    uont = (Use_Ontology)selNode.getUserObject();
    fullPrefix = uont.getPrefix();
    parent = (LabeledTreeNode)selNode.getParent();
    while (parent != null) {
      uont = (Use_Ontology)parent.getUserObject();
      prefix = uont.getPrefix();
      fullPrefix = prefix + "." + fullPrefix;
      parent = (LabeledTreeNode)parent.getParent();
    }
    preText.setText(fullPrefix);  
  }


  String getFullPrefix(String str) {

    int pend;

    pend = str.lastIndexOf('.');
    if (pend < 0)
      return null;
    else
      return str.substring(0, pend);
  }

  String removeFullPrefix(String str) {

    int pend;

    pend = str.lastIndexOf('.');
    if (pend < 0)
      return str;
    else
      return str.substring(pend + 1, str.length());
  }


  public void notifyUser(String s) {

    Notification note;
    
    // String text[] = {s};
    // String buttons[] = {"OK"};
    // note = new Notification(parent, "Error", true, text, buttons);
    note = new Notification(parent, "Error", s);
    note.showWorkaround();
  }

}
